package ru.dbotthepony.mc.otm.capability;

import net.minecraft.nbt.CompoundTag;
import net.minecraftforge.common.util.INBTSerializable;
import ru.dbotthepony.mc.otm.matter.MatterRegistry;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.math.BigDecimal;

public class MatterHandlerCapability implements IMatterHandler, INBTSerializable<CompoundTag> {
	protected final Runnable listener;
	protected BigDecimal stored = BigDecimal.ZERO;
	protected MatterDirection direction;
	protected BigDecimal max_storage;

	protected BigDecimal max_receive;
	protected BigDecimal max_extract;

	public MatterHandlerCapability(
		@Nullable Runnable listener,
		MatterDirection direction,
		BigDecimal max_storage
	) {
		this.listener = listener;
		this.direction = direction;
		this.max_storage = max_storage;
	}

	public MatterHandlerCapability(
		@Nullable Runnable listener,
		MatterDirection direction,
		BigDecimal max_storage,
		BigDecimal max_receive,
		BigDecimal max_extract
	) {
		this(listener, direction, max_storage);

		this.max_receive = max_receive;
		this.max_extract = max_extract;
	}

	@Nonnull
	@Override
	public BigDecimal getStoredMatter() {
		return stored;
	}

	@Nonnull
	@Override
	public BigDecimal getMaxStoredMatter() {
		return max_storage;
	}

	public boolean canReceiveAll(BigDecimal value) {
		return max_storage.compareTo(value) >= 0 && stored.add(value).compareTo(max_storage) <= 0;
	}

	@Nonnull
	@Override
	public BigDecimal receiveMatterOuter(BigDecimal howMuch, boolean simulate) {
		if (direction == MatterDirection.EXTRACT)
			return BigDecimal.ZERO;

		return receiveMatterInner(howMuch, simulate);
	}

	@Nonnull
	@Override
	public BigDecimal receiveMatterInner(BigDecimal howMuch, boolean simulate) {
		BigDecimal new_matter;

		if (max_receive == null) {
			new_matter = stored.add(howMuch, MatterRegistry.ROUND_RULES).min(max_storage);
		} else {
			new_matter = stored.add(howMuch.min(max_receive), MatterRegistry.ROUND_RULES).min(max_storage);
		}

		BigDecimal diff = new_matter.subtract(stored);

		if (!simulate && new_matter.compareTo(stored) != 0) {
			stored = new_matter;

			if (listener != null) {
				listener.run();
			}
		}

		return diff;
	}

	@Nonnull
	@Override
	public BigDecimal extractMatterOuter(BigDecimal howMuch, boolean simulate) {
		if (direction == MatterDirection.RECEIVE)
			return BigDecimal.ZERO;

		return extractMatterInner(howMuch, simulate);
	}

	@Nonnull
	@Override
	public BigDecimal extractMatterInner(BigDecimal howMuch, boolean simulate) {
		BigDecimal new_matter;

		if (max_receive == null) {
			new_matter = stored.subtract(howMuch, MatterRegistry.ROUND_RULES).max(BigDecimal.ZERO);
		} else {
			new_matter = stored.subtract(howMuch.min(max_receive), MatterRegistry.ROUND_RULES).max(BigDecimal.ZERO);
		}

		BigDecimal diff = stored.subtract(new_matter);

		if (!simulate && new_matter.compareTo(stored) != 0) {
			stored = new_matter;

			if (listener != null) {
				listener.run();
			}
		}

		return diff;
	}

	@Nonnull
	@Override
	public MatterDirection getDirection() {
		return direction;
	}

	@Override
	public CompoundTag serializeNBT() {
		CompoundTag tag = new CompoundTag();
		tag.putString("stored", stored.toString());
		tag.putString("max_storage", max_storage.toString());

		if (max_receive != null)
			tag.putString("max_receive", max_receive.toString());

		if (max_extract != null)
			tag.putString("max_extract", max_extract.toString());

		return tag;
	}

	@Override
	public void deserializeNBT(CompoundTag tag) {
		if (tag.contains("stored"))
			stored = new BigDecimal(tag.getString("stored"));

		if (tag.contains("max_storage"))
			max_storage = new BigDecimal(tag.getString("max_storage"));

		if (tag.contains("max_receive"))
			max_receive = new BigDecimal(tag.getString("max_receive"));
		else
			max_receive = null;

		if (tag.contains("max_extract"))
			max_extract = new BigDecimal(tag.getString("max_extract"));
		else
			max_extract = null;
	}
}
